@@ -2,44 +2,36 @@ import typing
 import warnings
 from copy import deepcopy
 
-from marshmallow import (
-    RAISE,
-    Schema,
-    ValidationError,
-    missing as missing_,
-    post_load,
-    pre_dump,
-    pre_load,
-    validates_schema,
-)
+from marshmallow import RAISE, ValidationError, missing as missing_, post_load, pre_dump, pre_load, validates_schema
 
 from bioimageio.spec.rdf import v0_2 as rdf
 from bioimageio.spec.shared import field_validators, fields
 from bioimageio.spec.shared.common import get_args, get_args_flat
-from bioimageio.spec.shared.schema import SharedBioImageIOSchema
+from bioimageio.spec.shared.schema import (
+    ImplicitOutputShape,
+    ParametrizedInputShape,
+    SharedBioImageIOSchema,
+    SharedProcessingSchema,
+)
+from bioimageio.spec.shared.utils import get_ref_url
 from . import raw_nodes
 
 Author = rdf.schema.Author
 CiteEntry = rdf.schema.CiteEntry
 
 
-class BioImageIOSchema(SharedBioImageIOSchema):
+class _BioImageIOSchema(SharedBioImageIOSchema):
     raw_nodes = raw_nodes
 
 
-class RunMode(BioImageIOSchema):
+class RunMode(_BioImageIOSchema):
     name = fields.String(
         required=True, bioimageio_description="The name of the `run_mode`"
     )  # todo: limit valid run mode names
     kwargs = fields.Kwargs()
 
 
-class SpecWithKwargs(BioImageIOSchema):
-    spec: fields.SpecURI
-    kwargs = fields.Kwargs()
-
-
-class Tensor(BioImageIOSchema):
+class _TensorBase(_BioImageIOSchema):
     name = fields.String(
         required=True, validate=field_validators.Predicate("isidentifier"), bioimageio_description="Tensor name."
     )
@@ -86,30 +78,51 @@ class Tensor(BioImageIOSchema):
                 raise ValidationError("`kwargs.axes` needs to be subset of axes")
 
 
-class Processing(BioImageIOSchema):
-    class Binarize(BioImageIOSchema):
-        # todo: inherit from a "TransformSchema" that allows generation of docs for pre and postprocessing?
-        threshold = fields.Float(required=True)
+class Processing(_BioImageIOSchema):
+    class binarize(SharedProcessingSchema):
+        bioimageio_description = (
+            "Binarize the tensor with a fixed threshold, values above the threshold will be set to one, values below "
+            "the threshold to zero."
+        )
+        threshold = fields.Float(required=True, bioimageio_description="The fixed threshold")
 
-    class Clip(BioImageIOSchema):
-        min = fields.Float(required=True)
-        max = fields.Float(required=True)
+    class clip(SharedProcessingSchema):
+        bioimageio_description = "Set tensor values below min to min and above max to max."
+        min = fields.Float(required=True, bioimageio_description="minimum value for clipping")
+        max = fields.Float(required=True, bioimageio_description="maximum value for clipping")
 
-    class ScaleLinear(BioImageIOSchema):
-        axes = fields.Axes(required=True, valid_axes="czyx")
-        gain = fields.Array(fields.Float(), missing=fields.Float(missing=1.0))  # todo: check if gain match input axes
+    class scale_linear(SharedProcessingSchema):
+        bioimageio_description = "Fixed linear scaling."
+        axes = fields.Axes(
+            required=True,
+            valid_axes="czyx",
+            bioimageio_description="The subset of axes to scale jointly. "
+            "For example xy to scale the two image axes for 2d data jointly. "
+            "The batch axis (b) is not valid here.",
+        )
+        gain = fields.Array(
+            fields.Float(), missing=fields.Float(missing=1.0), bioimageio_description="multiplicative factor"
+        )  # todo: check if gain match input axes
         offset = fields.Array(
-            fields.Float(), missing=fields.Float(missing=0.0)
+            fields.Float(), missing=fields.Float(missing=0.0), bioimageio_description="additive term"
         )  # todo: check if offset match input axes
 
         @validates_schema
         def either_gain_or_offset(self, data, **kwargs):
-            if data["gain"] == 1.0 and data["offset"] == 0:
+            gain = data["gain"]
+            if isinstance(gain, (float, int)):
+                gain = [gain]
+
+            offset = data["offset"]
+            if isinstance(offset, (float, int)):
+                offset = [offset]
+
+            if all(g == 1.0 for g in gain) and all(off == 0 for off in offset):
                 raise ValidationError("Specify gain!=1.0 or offset!=0.0")
 
     @validates_schema
     def kwargs_match_selected_preprocessing_name(self, data, **kwargs):
-        schema_name = "".join(word.title() for word in data["name"].split("_"))
+        schema_name = data["name"]
 
         try:
             schema_class = getattr(self, schema_name)
@@ -122,21 +135,39 @@ class Processing(BioImageIOSchema):
         if kwargs_validation_errors:
             raise ValidationError(f"Invalid `kwargs` for '{data['name']}': {kwargs_validation_errors}")
 
-    class Sigmoid(BioImageIOSchema):
-        pass
+    class sigmoid(SharedProcessingSchema):
+        bioimageio_description = ""
 
-    class ZeroMeanUnitVariance(BioImageIOSchema):
+    class zero_mean_unit_variance(SharedProcessingSchema):
+        bioimageio_description = "Subtract mean and divide by variance."
         mode = fields.ProcMode(required=True)
-        axes = fields.Axes(required=True, valid_axes="czyx")
-        mean = fields.Array(fields.Float())  # todo: check if means match input axes (for mode 'fixed')
-        std = fields.Array(fields.Float())
-        eps = fields.Float(missing=1e-6)
+        axes = fields.Axes(
+            required=True,
+            valid_axes="czyx",
+            bioimageio_description="The subset of axes to normalize jointly. For example xy to normalize the two image "
+            "axes for 2d data jointly. The batch axis (b) is not valid here.",
+        )
+        mean = fields.Array(
+            fields.Float(),
+            bioimageio_description="The mean value(s) to use for `mode == fixed`. For example `[1.1, 2.2, 3.3]` in the "
+            "case of a 3 channel image where the channels are not normalized jointly.",
+        )  # todo: check if means match input axes (for mode 'fixed')
+        std = fields.Array(
+            fields.Float(),
+            bioimageio_description="The standard deviation values to use for `mode == fixed`. Analogous to mean.",
+        )
+        eps = fields.Float(
+            missing=1e-6,
+            bioimageio_description="epsilon for numeric stability: `out = (tensor - mean) / (std + eps)`. "
+            "Default value: 10^-6.",
+        )
 
         @validates_schema
         def mean_and_std_match_mode(self, data, **kwargs):
             if data["mode"] == "fixed" and ("mean" not in data or "std" not in data):
                 raise ValidationError(
-                    "`kwargs` for 'zero_mean_unit_variance' preprocessing with `mode` 'fixed' require additional `kwargs`: `mean` and `std`."
+                    "`kwargs` for 'zero_mean_unit_variance' preprocessing with `mode` 'fixed' require additional "
+                    "`kwargs`: `mean` and `std`."
                 )
             elif data["mode"] != "fixed" and ("mean" in data or "std" in data):
                 raise ValidationError(
@@ -148,22 +179,41 @@ class Preprocessing(Processing):
     name = fields.String(
         required=True,
         validate=field_validators.OneOf(get_args(raw_nodes.PreprocessingName)),
-        bioimageio_description=f"Name of preprocessing. One of: {', '.join(get_args(raw_nodes.PreprocessingName))} "
-        f"(see [supported_formats_and_operations.md#preprocessing](https://github.com/bioimage-io/configuration/"
-        f"blob/master/supported_formats_and_operations.md#preprocessing) "
-        f"for information on which transformations are supported by specific consumer software).",
+        bioimageio_description=f"Name of preprocessing. One of: {', '.join(get_args(raw_nodes.PreprocessingName))}.",
+    )
+    kwargs = fields.Kwargs(
+        bioimageio_description=f"Key word arguments as described in [preprocessing spec]"
+        f"(https://github.com/bioimage-io/spec-bioimage-io/blob/gh-pages/preprocessing_spec_"
+        f"{'_'.join(get_args(raw_nodes.FormatVersion)[-1].split('.')[:2])}.md)."
     )
-    kwargs = fields.Kwargs()
 
-    class ScaleRange(Schema):
+    class scale_range(SharedProcessingSchema):
+        bioimageio_description = "Scale with percentiles."
         mode = fields.ProcMode(required=True, valid_modes=("per_dataset", "per_sample"))
-        axes = fields.Axes(required=True, valid_axes="czyx")
+        axes = fields.Axes(
+            required=True,
+            valid_axes="czyx",
+            bioimageio_description="The subset of axes to normalize jointly. For example xy to normalize the two image "
+            "axes for 2d data jointly. The batch axis (b) is not valid here.",
+        )
         min_percentile = fields.Float(
-            default=0, validate=field_validators.Range(0, 100, min_inclusive=True, max_inclusive=False)
+            default=0,
+            validate=field_validators.Range(0, 100, min_inclusive=True, max_inclusive=False),
+            bioimageio_description="The lower percentile used for normalization, in range 0 to 100. Default value: 0.",
         )
         max_percentile = fields.Float(
-            default=100, validate=field_validators.Range(1, 100, min_inclusive=False, max_inclusive=True)
-        )  # as a precaution 'max_percentile' needs to be greater than 1
+            default=100,
+            validate=field_validators.Range(1, 100, min_inclusive=False, max_inclusive=True),
+            bioimageio_description="The upper percentile used for normalization, in range 1 to 100. Has to be bigger "
+            "than min_percentile. Default value: 100. The range is 1 to 100 instead of 0 to 100 to avoid mistakenly "
+            "accepting percentiles specified in the range 0.0 to 1.0.",
+        )
+        eps = fields.Float(
+            missing=1e-6,
+            bioimageio_description="Epsilon for numeric stability: "
+            "`out = (tensor - v_lower) / (v_upper - v_lower + eps)`; "
+            "with `v_lower,v_upper` values at the respective percentiles. Default value: 10^-6.",
+        )
 
         @validates_schema
         def min_smaller_max(self, data, **kwargs):
@@ -177,25 +227,48 @@ class Postprocessing(Processing):
     name = fields.String(
         validate=field_validators.OneOf(get_args(raw_nodes.PostprocessingName)),
         required=True,
-        bioimageio_description=f"Name of postprocessing. One of: {', '.join(get_args(raw_nodes.PostprocessingName))} "
-        f"(see [supported_formats_and_operations.md#postprocessing](https://github.com/bioimage-io/configuration/"
-        f"blob/master/supported_formats_and_operations.md#postprocessing) "
-        f"for information on which transformations are supported by specific consumer software).",
+        bioimageio_description=f"Name of postprocessing. One of: {', '.join(get_args(raw_nodes.PostprocessingName))}.",
+    )
+    kwargs = fields.Kwargs(
+        bioimageio_description=f"Key word arguments as described in [postprocessing spec]"
+        f"(https://github.com/bioimage-io/spec-bioimage-io/blob/gh-pages/postprocessing_spec_"
+        f"{'_'.join(get_args(raw_nodes.FormatVersion)[-1].split('.')[:2])}.md)."
     )
-    kwargs = fields.Kwargs()
 
-    class ScaleRange(Preprocessing.ScaleRange):
-        reference_tensor = fields.String(required=False, validate=field_validators.Predicate("isidentifier"))
+    class scale_range(Preprocessing.scale_range):
+        reference_tensor = fields.String(
+            required=False,
+            validate=field_validators.Predicate("isidentifier"),
+            bioimageio_description="Tensor name to compute the percentiles from. Default: The tensor itself. "
+            "If mode==per_dataset this needs to be the name of an input tensor.",
+        )
 
-    class ScaleMeanVariance(BioImageIOSchema):
+    class scale_mean_variance(SharedProcessingSchema):
+        bioimageio_description = "Scale the tensor s.t. its mean and variance match a reference tensor."
         mode = fields.ProcMode(required=True, valid_modes=("per_dataset", "per_sample"))
-        reference_tensor = fields.String(required=True, validate=field_validators.Predicate("isidentifier"))
+        reference_tensor = fields.String(
+            required=True,
+            validate=field_validators.Predicate("isidentifier"),
+            bioimageio_description="Name of tensor to match.",
+        )
 
 
-class InputTensor(Tensor):
-    shape = fields.InputShape(required=True, bioimageio_description="Specification of tensor shape.")
+class InputTensor(_TensorBase):
+    shape = fields.Union(
+        [
+            fields.ExplicitShape(
+                bioimageio_description="Exact shape with same length as `axes`, e.g. `shape: [1, 512, 512, 1]`"
+            ),
+            fields.Nested(
+                ParametrizedInputShape(),
+                bioimageio_description="A sequence of valid shapes given by `shape = min + k * step for k in {0, 1, ...}`.",
+            ),
+        ],
+        required=True,
+        bioimageio_description="Specification of input tensor shape.",
+    )
     preprocessing = fields.List(
-        fields.Nested(Preprocessing), bioimageio_description="Description of how this input should be preprocessed."
+        fields.Nested(Preprocessing()), bioimageio_description="Description of how this input should be preprocessed."
     )
     processing_name = "preprocessing"
 
@@ -228,17 +301,31 @@ class InputTensor(Tensor):
             raise ValidationError("Input shape has to be 1 in the batch dimension b.")
 
 
-class OutputTensor(Tensor):
-    shape = fields.OutputShape(required=True)
+class OutputTensor(_TensorBase):
+    shape = fields.Union(
+        [
+            fields.ExplicitShape(),
+            fields.Nested(
+                ImplicitOutputShape(),
+                bioimageio_description="In reference to the shape of an input tensor, the shape of the output "
+                "tensor is `shape = shape(input_tensor) * scale + 2 * offset`.",
+            ),
+        ],
+        required=True,
+        bioimageio_description="Specification of output tensor shape.",
+    )
     halo = fields.List(
-        fields.Integer,
-        bioimageio_description="The halo to crop from the output tensor (for example to crop away boundary effects or "
-        "for tiling). The halo should be cropped from both sides, i.e. `shape_after_crop = shape - 2 * halo`. The "
-        "`halo` is not cropped by the bioimage.io model, but is left to be cropped by the consumer software. Use "
-        "`shape:offset` if the model output itself is cropped and input and output shapes not fixed.",
+        fields.Integer(),
+        bioimageio_description="Hint to describe the potentially corrupted edge region of the output tensor, due to "
+        "boundary effects. "
+        "The `halo` is not cropped by the bioimage.io model, but is left to be cropped by the consumer software. "
+        f"An example implementation of prediction with tiling, accounting for the halo can be found [here]("
+        f"{get_ref_url('function', '_predict_with_tiling_impl', 'https://github.com/bioimage-io/core-bioimage-io-python/blob/main/bioimageio/core/prediction.py')}). "
+        "Use `shape:offset` if the model output itself is cropped and input and output shapes not fixed. ",
     )
     postprocessing = fields.List(
-        fields.Nested(Postprocessing), bioimageio_description="Description of how this output should be postprocessed."
+        fields.Nested(Postprocessing()),
+        bioimageio_description="Description of how this output should be postprocessed.",
     )
     processing_name = "postprocessing"
 
@@ -261,32 +348,31 @@ _common_sha256_hint = (
     "Or you can generate the SHA256 code for your model and weights by using for example, `hashlib` in Python. "
     # "[here is a codesnippet](#code-snippet-to-compute-sha256-checksum)."  # todo: link to code snippet and don't multiply it
     + """
-Code snippet to compute SHA256 checksum
-
-```python
-import hashlib
-
-filename = "your filename here"
-with open(filename, "rb") as f:
-  bytes = f.read() # read entire file as bytes
-  readable_hash = hashlib.sha256(bytes).hexdigest()
-  print(readable_hash)
-  ```
+    Code snippet to compute SHA256 checksum
+    
+    ```python
+    import hashlib
+    
+    filename = "your filename here"
+    with open(filename, "rb") as f:
+      bytes = f.read() # read entire file as bytes
+      readable_hash = hashlib.sha256(bytes).hexdigest()
+      print(readable_hash)
+      ```
 
 """
 )
 
 
-class WeightsEntryBase(BioImageIOSchema):
-    weights_format: fields.String
+class _WeightsEntryBase(_BioImageIOSchema):
     authors = fields.List(
-        fields.Nested(Author),
+        fields.Nested(Author()),
         bioimageio_description="A list of authors. If this is the root weight (it does not have a `parent` field): the "
         "person(s) that have trained this model. If this is a child weight (it has a `parent` field): the person(s) "
         "who have converted the weights to this format.",
     )  # todo: copy root authors if missing
     attachments = fields.Dict(
-        fields.String,
+        fields.String(),
         fields.List(fields.Union([fields.URI(), fields.Raw()])),
         bioimageio_description="Dictionary of text keys and list values (that may contain any valid yaml) to "
         "additional, relevant files that are specific to the current weight format. A list of URIs can be listed under"
@@ -302,7 +388,11 @@ class WeightsEntryBase(BioImageIOSchema):
         validate=field_validators.Length(equal=64),
         bioimageio_description="SHA256 checksum of the source file specified. " + _common_sha256_hint,
     )
-    source = fields.URI(required=True, bioimageio_description="Link to the source file. Preferably a url.")
+    source = fields.Union(
+        [fields.URI(), fields.RelativeLocalPath()],
+        required=True,
+        bioimageio_description="URI or path to the weights file. Preferably a url.",
+    )
     weights_format = fields.String(
         validate=field_validators.OneOf(get_args(raw_nodes.WeightsFormat)), required=True, load_only=True
     )
@@ -326,35 +416,35 @@ class WeightsEntryBase(BioImageIOSchema):
         return raw_node
 
 
-class KerasHdf5WeightsEntry(WeightsEntryBase):
+class KerasHdf5WeightsEntry(_WeightsEntryBase):
     bioimageio_description = "Keras HDF5 weights format"
     weights_format = fields.String(validate=field_validators.Equal("keras_hdf5"), required=True, load_only=True)
     tensorflow_version = fields.StrictVersion()  # todo: required=True
 
 
-class OnnxWeightsEntry(WeightsEntryBase):
+class OnnxWeightsEntry(_WeightsEntryBase):
     bioimageio_description = "ONNX weights format"
     weights_format = fields.String(validate=field_validators.Equal("onnx"), required=True, load_only=True)
     opset_version = fields.Integer()  # todo: required=True
 
 
-class PytorchStateDictWeightsEntry(WeightsEntryBase):
+class PytorchStateDictWeightsEntry(_WeightsEntryBase):
     bioimageio_description = "PyTorch state dictionary weights format"
     weights_format = fields.String(validate=field_validators.Equal("pytorch_state_dict"), required=True, load_only=True)
 
 
-class PytorchScriptWeightsEntry(WeightsEntryBase):
+class PytorchScriptWeightsEntry(_WeightsEntryBase):
     bioimageio_description = "Torch Script weights format"
     weights_format = fields.String(validate=field_validators.Equal("pytorch_script"), required=True, load_only=True)
 
 
-class TensorflowJsWeightsEntry(WeightsEntryBase):
+class TensorflowJsWeightsEntry(_WeightsEntryBase):
     bioimageio_description = "Tensorflow Javascript weights format"
     weights_format = fields.String(validate=field_validators.Equal("tensorflow_js"), required=True, load_only=True)
     tensorflow_version = fields.StrictVersion()  # todo: required=True
 
 
-class TensorflowSavedModelBundleWeightsEntry(WeightsEntryBase):
+class TensorflowSavedModelBundleWeightsEntry(_WeightsEntryBase):
     bioimageio_description = "Tensorflow Saved Model Bundle weights format"
     weights_format = fields.String(
         validate=field_validators.Equal("tensorflow_saved_model_bundle"), required=True, load_only=True
@@ -372,13 +462,14 @@ WeightsEntry = typing.Union[
 ]
 
 
-class ModelParent(BioImageIOSchema):
-    uri = fields.URI(
+class ModelParent(_BioImageIOSchema):
+    uri = fields.Union(  # todo: allow URI or DOI instead (and not local path!?)
+        [fields.URI(), fields.RelativeLocalPath()],
         bioimageio_description="Url of another model available on bioimage.io or path to a local model in the "
         "bioimage.io specification. If it is a url, it needs to be a github url linking to the page containing the "
-        "model (NOT the raw file)."
+        "model (NOT the raw file).",
     )
-    sha256 = fields.SHA256(bioimageio_description="Hash of the weights of the parent model.")
+    sha256 = fields.SHA256(bioimageio_description="Hash of the parent model RDF.")
 
 
 class Model(rdf.schema.RDF):
@@ -397,18 +488,35 @@ _optional*_ with an asterisk indicates the field is optional depending on the va
 """
     # todo: unify authors with RDF (optional or required?)
     authors = fields.List(
-        fields.Nested(Author), required=True, bioimageio_description=rdf.schema.RDF.authors_bioimageio_description
+        fields.Nested(Author()), required=True, bioimageio_description=rdf.schema.RDF.authors_bioimageio_description
     )
 
-    badges = missing_  # todo: allow badges for Model (RDF has it)
-    cite = fields.Nested(
-        CiteEntry,
-        many=True,
+    badges = missing_
+    cite = fields.List(
+        fields.Nested(CiteEntry()),
         required=True,  # todo: unify authors with RDF (optional or required?)
         bioimageio_description=rdf.schema.RDF.cite_bioimageio_description,
     )
 
-    download_url = missing_  # todo: allow download_url for Model (RDF has it)
+    documentation = fields.Union(
+        [
+            fields.URL(),
+            fields.RelativeLocalPath(
+                validate=field_validators.Attribute(
+                    "suffix",
+                    field_validators.Equal(
+                        ".md", error="{!r} is invalid; expected markdown file with '.md' extension."
+                    ),
+                )
+            ),
+        ],
+        required=True,
+        bioimageio_description="Relative path to file with additional documentation in markdown. This means: 1) only "
+        "relative file path is allowed 2) the file must be in markdown format with `.md` file name extension 3) URL is "
+        "not allowed. It is recommended to use `README.md` as the documentation name.",
+    )
+
+    download_url = missing_
 
     dependencies = fields.Dependencies(  # todo: add validation (0.4.0?)
         bioimageio_description="Dependency manager and dependency file, specified as `<dependency manager>:<relative "
@@ -439,7 +547,7 @@ is in an unsupported format version. The current format version described here i
         + "(which contains the configuration yaml file) should be used.",
     )
 
-    icon = missing_  # todo: allow icon for Model (RDF has it)
+    icon = missing_
 
     kwargs = fields.Kwargs(
         bioimageio_description="Keyword arguments for the implementation specified by `source`. "
@@ -466,24 +574,23 @@ is in an unsupported format version. The current format version described here i
     )
 
     packaged_by = fields.List(
-        fields.Nested(Author),
+        fields.Nested(Author()),
         bioimageio_description=f"The persons that have packaged and uploaded this model. Only needs to be specified if "
         f"different from `authors` in root or any entry in `weights`.",
     )
 
     parent = fields.Nested(
-        ModelParent,
+        ModelParent(),
         bioimageio_description="Parent model from which the trained weights of this model have been derived, e.g. by "
         "finetuning the weights of this model on a different dataset. For format changes of the same trained model "
         "checkpoint, see `weights`.",
     )
 
     run_mode = fields.Nested(
-        RunMode,
+        RunMode(),
         bioimageio_description="Custom run mode for this model: for more complex prediction procedures like test time "
-        "data augmentation that currently cannot be expressed in the specification. The different run modes should be "
-        "listed in [supported_formats_and_operations.md#Run Modes]"
-        "(https://github.com/bioimage-io/configuration/blob/master/supported_formats_and_operations.md#run-modes).",
+        "data augmentation that currently cannot be expressed in the specification. "
+        "No standard run modes are defined yet.",
     )
 
     sha256 = fields.String(
@@ -513,11 +620,11 @@ is in an unsupported format version. The current format version described here i
             validate=field_validators.OneOf(get_args(raw_nodes.WeightsFormat)),
             required=True,
             bioimageio_description=f"Format of this set of weights. Weight formats can define additional (optional or "
-            f"required) fields. See [supported_formats_and_operations.md#Weight Format]"
-            f"(https://github.com/bioimage-io/configuration/blob/master/supported_formats_and_operations.md#weight_format). "
+            f"required) fields. See [weight_formats_spec_0_3.md]"
+            f"(https://github.com/bioimage-io/spec-bioimage-io/blob/gh-pages/weight_formats_spec_0_3.md). "
             f"One of: {', '.join(get_args(raw_nodes.WeightsFormat))}",
         ),
-        fields.Union([fields.Nested(we) for we in get_args(WeightsEntry)]),
+        fields.Union([fields.Nested(we()) for we in get_args(WeightsEntry)]),
         required=True,
         bioimageio_description="The weights for this model. Weights can be given for different formats, but should "
         "otherwise be equivalent. The available weight formats determine which consumers can use this model.",
@@ -537,52 +644,63 @@ is in an unsupported format version. The current format version described here i
 
         return data
 
-    inputs = fields.Nested(
-        InputTensor, many=True, bioimageio_description="Describes the input tensors expected by this model."
+    inputs = fields.List(
+        fields.Nested(InputTensor()), bioimageio_description="Describes the input tensors expected by this model."
     )
-    outputs = fields.Nested(
-        OutputTensor, many=True, bioimageio_description="Describes the output tensors from this model."
+    outputs = fields.List(
+        fields.Nested(OutputTensor()), bioimageio_description="Describes the output tensors from this model."
     )
 
     test_inputs = fields.List(
-        fields.URI,
+        fields.Union([fields.URI(), fields.RelativeLocalPath()]),
         required=True,
-        bioimageio_description="List of URIs to test inputs as described in inputs for a single test case. "
-        "Supported file formats/extensions: '.npy'",
+        bioimageio_description="List of URIs or local relative paths to test inputs as described in inputs for "
+        "**a single test case**. "
+        "This means if your model has more than one input, you should provide one URI for each input."
+        "Each test input should be a file with a ndarray in "
+        "[numpy.lib file format](https://numpy.org/doc/stable/reference/generated/numpy.lib.format.html#module-numpy.lib.format)."
+        "The extension must be '.npy'.",
+    )
+    test_outputs = fields.List(
+        fields.Union([fields.URI(), fields.RelativeLocalPath()]),
+        required=True,
+        bioimageio_description="Analog to to test_inputs.",
     )
-    test_outputs = fields.List(fields.URI, required=True, bioimageio_description="Analog to to test_inputs.")
 
     sample_inputs = fields.List(
-        fields.URI,
-        bioimageio_description="List of URIs to sample inputs to illustrate possible inputs for the model, for example "
-        "stored as png or tif images.",
+        fields.Union([fields.URI(), fields.RelativeLocalPath()]),
+        bioimageio_description="List of URIs/local relative paths to sample inputs to illustrate possible inputs for "
+        "the model, for example stored as png or tif images. "
+        "The model is not tested with these sample files that serve to inform a human user about an example use case.",
     )
     sample_outputs = fields.List(
-        fields.URI, bioimageio_description="List of URIs to sample outputs corresponding to the `sample_inputs`."
+        fields.Union([fields.URI(), fields.RelativeLocalPath()]),
+        bioimageio_description="List of URIs/local relative paths to sample outputs corresponding to the "
+        "`sample_inputs`.",
     )
 
-    config = fields.Dict(
+    config = fields.YamlDict(
         bioimageio_description=rdf.schema.RDF.config_bioimageio_description
         + """
 
-For example:
-```yaml
-config:
-  # custom config for DeepImageJ, see https://github.com/bioimage-io/configuration/issues/23
-  deepimagej:
-    model_keys:
-      # In principle the tag "SERVING" is used in almost every tf model
-      model_tag: tf.saved_model.tag_constants.SERVING
-      # Signature definition to call the model. Again "SERVING" is the most general
-      signature_definition: tf.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY
-    test_information:
-      input_size: [2048x2048] # Size of the input images
-      output_size: [1264x1264 ]# Size of all the outputs
-      device: cpu # Device used. In principle either cpu or GPU
-      memory_peak: 257.7 Mb # Maximum memory consumed by the model in the device
-      runtime: 78.8s # Time it took to run the model
-      pixel_size: [9.658E-4µmx9.658E-4µm] # Size of the pixels of the input
-```
+    For example:
+    ```yaml
+    config:
+      # custom config for DeepImageJ, see https://github.com/bioimage-io/configuration/issues/23
+      deepimagej:
+        model_keys:
+          # In principle the tag "SERVING" is used in almost every tf model
+          model_tag: tf.saved_model.tag_constants.SERVING
+          # Signature definition to call the model. Again "SERVING" is the most general
+          signature_definition: tf.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY
+        test_information:
+          input_size: [2048x2048] # Size of the input images
+          output_size: [1264x1264 ]# Size of all the outputs
+          device: cpu # Device used. In principle either cpu or GPU
+          memory_peak: 257.7 Mb # Maximum memory consumed by the model in the device
+          runtime: 78.8s # Time it took to run the model
+          pixel_size: [9.658E-4µmx9.658E-4µm] # Size of the pixels of the input
+    ```
 """
     )
 
@@ -640,7 +758,7 @@ config:
 
     @validates_schema
     def weights_entries_match_weights_formats(self, data, **kwargs):
-        weights: typing.Dict[str, WeightsEntryBase] = data["weights"]
+        weights: typing.Dict[str, _WeightsEntryBase] = data["weights"]
         for weights_format, weights_entry in weights.items():
             if weights_format in ["keras_hdf5", "tensorflow_js", "tensorflow_saved_model_bundle"]:
                 assert isinstance(
