@@ -1,23 +1,37 @@
-import json
 import sys
 from argparse import ArgumentParser
 from pathlib import Path
 
-from marshmallow_jsonschema import JSONSchema
+try:
+    from ruamel.yaml import YAML
 
-import bioimageio.spec
-
-from bioimageio.spec.shared.common import ruamel_yaml, pyyaml_yaml
-
-if ruamel_yaml is None:
+    ruamel_yaml = YAML(typ="safe")
+except ImportError:
     raise RuntimeError("Cannot compare yaml syntax without the ruamel.yaml package")
 
+try:
+    import yaml as _pyyaml_yaml
+
+    class pyyaml_yaml:
+        @staticmethod
+        def load(path: Path):
+            with path.open() as f:
+                return _pyyaml_yaml.load(f, None)  # fmt: off
+
+
+except ImportError:
+    raise RuntimeError("Cannot compare yaml syntax without the pyyaml package")
+
 
 def parse_args():
     p = ArgumentParser(
         description="Check for differences between yaml 1.1 (using PyYAML) and yaml 1.2 syntax (using ruamel.yaml)."
     )
-    p.add_argument("resource_description_path", type=Path)
+    p.add_argument(
+        "--resource_description_path",
+        type=Path,
+        default=Path(__file__).parent / "../example_specs/models/unet2d_nuclei_broad/rdf.yaml",
+    )
     args = p.parse_args()
     return args
 
